RegExpBuilder
=============
RegExpBuilder integrates regular expressions into the programming language, thereby making them easy to read and maintain. Regular Expressions are created by using chained methods and variables such as arrays or strings.

<h2>Installation</h2>
Here are a couple of examples using Javascript:

<h3>Money</h3>

```
import { find } from 'regexpbuilder';

const regex = find('$')
  .min(1).digits()
  .then('.')
  .digit()
  .digit();
  
regex.test('$10.00'); // true
```

<h3>Nested patterns</h3>

```
import { min, just } from 'regexpbuilder';

const pattern = min(1).of('p')
  .min(2).of('q');

const regex = just(2).like(pattern);

regex.test('pqqpqq'); // true
```

<h2>API documentation</h2>

<h3>start()</h3>

Matches the start of a line or the start of input depending on whether multiline is set.

```
const regex = start().then('the');

regex.test('the bear has claws.'); // true
regex.test('bears know where the honey is.'); // false
```

<h3>end()</h3>

Matches the end of a line or the end of input depending on whether multiline is set.

```
const regex = find('house').end();

regex.test('The house'); // true
regex.test('The house is on the hill'); // false
```

<h3>boundary()</h3>

Matches a word boundary. This is the place before or after a word.

```
const regex = find('cure').boundary();

regex.test('cure.'); // true
regex.test('The cure is a band.'); // true
regex.test('Thecureisaband'); // false
```

<h3>nonBoundary()</h3>

The opposite of boundary().

<h3>find(pattern)</h3>

pattern: either a string or a RegExpBuilder object

Matches the string or pattern once.

```
const money = find('$').min(1).digits();

const regex = find(money)
  .then(' ')
  .then('per hour');

regex.test('$100 per hour'); // true
```

<h3>then(pattern)</h3>

An alias for find(). It exists for readability.

<h3>maybe(pattern)</h3>

pattern: either a string or a RegExpBuilder object

Matches the string or pattern at most once.

```
const regex = find('http')
  .maybe('s')
  .then(' is a protocol');

regex.test('http is a protocol'); // true
regex.test('https is a protocol'); // true
```

<h3>either(pattern)</h3>

pattern: either a string or a RegExpBuilder object

Used with or(pattern).

```
const regex = either('house').or('apartment');

console.log(regex.test('house')); // true
```

<h3>or(pattern)</h3>

pattern: either a string or a RegExpBuilder object

Used with either(pattern).

```
const regex = either('house').or('apartment');

console.log(regex.test('house')); // true
```

<h3>just(count)</h3>

count: an integer

Matches the pattern exactly count number of times.

```
const regex = just(3).of('p')
  .just(2).from('pqr');

regex.test('ppprr'); // true
regex.test('pppprr'); // false
```

<h3>min(count)</h3>

count: an integer

Matches the pattern at least count number of times. Can be combined with max(count) to specify an exact range.

```
const regex = min(3).digits();

regex.test('1000'); // true
regex.test('9'); // false
```

<h3>max(count)</h3>

count: an integer

Matches the pattern at most count number of times. Can be combined with min(count) to specify an exact range.

```
const regex = min(3).max(5).letters();

regex.test('is'); // false
regex.test('household'); // false
regex.test('tree'); // true
```

<h3>of(text)</h3>

text: a string

Matches the supplied string. This method needs to be preceeded by min, max, or just.

```
const regex = just(2).of('20');

regex.test('2020'); // true
regex.test('2021'); // false
```

<h3>ofAny()</h3>

Matches any character. This method needs to be preceeded by min, max, or just.

```
const regex = find('<')
  .just(4).ofAny()
  .then('>');

regex.test('<body>'); // true
regex.test('<list>'); // true
regex.test('<li>'); // false
```

<h3>ofGroup(label)</h3>

label: a string used to refer to the group when needed

A reference to a group that was previously captured using asGroup(label).

```
const regex = just(1).letter().asGroup('letter')
  .then(' ')
  .just(1).ofGroup('letter');

regex.test('This seems'); // true
regex.test('Where is'); // false
```

<h3>from(chars)</h3>

chars: a string or an array. Ranges can also be used, such as 'p-rv', or ['p-r', 'v'], both of which will match p, q, r, and v.

Matches any character that is specified in the list of characters. This method must be preceeded by min, max, or just.

```
const regex = just(3).from('pqr');

regex.test('ppp'); // true
regex.test('prr'); // true
regex.test('prs'); // false
```

<h3>notFrom(chars)</h3>

The opposite of from(chars). Matches any character that is not in the list specified.

<h3>like(builder)</h3>

builder: a RegExpBuilder object

This method is used for nesting patterns.

```
const pattern = min(1).of('p')
  .min(2).of('q');

const regex = just(2).like(pattern);

regex.test('ppqqqpppqqqq'); // true
```

<h3>reluctantly()</h3>

Specifies that the pattern should be matched reluctantly, as opposed to greedily.

```
const regex = min(3).letters().reluctantly()
  .then(' hill');

console.log(regex.exec('The hill was a large hill')[0]); // The hill
```

<h3>ahead(pattern)</h3>

pattern: a string or RegExpBuilder object

A lookahead. It scans ahead for the pattern, but does not include that pattern in the matched result.

```
const regex = find('bus').ahead('load');

console.log(regex.exec('busload')[0]); // bus
```

<h3>notAhead(pattern)</h3>

The opposite of ahead(pattern).

<h3>behind(pattern)</h3>

pattern: a string or RegExpBuilder object

A lookbehind. It scans behind the current match to see if the pattern exists, but does not include that pattern in the matched results.

```
const regex = behind('<')
  .find('body')
  .ahead('>');

console.log(regex.exec('<body>')[0]); // body
```

<h3>notBehind</h3>

The opposite of behind(pattern).

<h3>asGroup(label)</h3>

label: a string

Captures the pattern and gives it a label.

```
const pattern = either('house').or('apartment');

const regex = find('The ')
  .then(pattern).asGroup('dwelling');

console.log(regex.exec('The house').groups.dwelling); // house
```

<h3>withProp(prop)</h3>

prop: a string that represents a unicode property

Matches a unicode character with the specified property. The unicode flag must be added to the regular expression for this to work.

```
const regex = min(2).withProp('Emoji')
  .unicode();

regex.test('There are no emojis here.'); // false
regex.test('This has emojis 😂😍'); // true
```

<h3>any()</h3>

Matches any character once. The singular version of ofAny(). It does not need to be preceeded by any quantity specifier.

```
const regex = any();

regex.test('q'); // true
regex.test('r'); // true
```

<h3>lineBreak()</h3>

Matches a line break. The singular version of lineBreaks(). It does not need to be preceeded by any quantity specifier.

<h3>lineBreaks()</h3>

Matches a line break. This method needs to be preceeded by min, max, or just.

<h3>whitespace()</h3>

Matches whitespace. This can be used by itself, or it can be preceeded by min, max, or just.

<h3>nonWhitespace()</h3>

The opposite of whitespace().

<h3>tab()</h3>

Matches a tab. It is the singular version of tabs().

<h3>tabs()</h3>

Matches a tab. This method needs to be preceeded by min, max, or just.

<h3>digit()</h3>

Matches a digit. It is the singular version of digits().

<h3>digits()</h3>

Matches a digit. This method needs to be preceeded by min, max, or just.

<h3>letter()</h3>

Matches a letter. This is the singular version of letters().

<h3>letters()</h3>

Matches a letter. This method needs to be preceeded by min, max, or just.

<h3>lowerCaseLetter()</h3>

Matches a lower case letter. This is the singular version of lowerCaseLetters().

<h3>lowerCaseLetters()</h3>

Matches a lower case letter. This method needs to be preceeded by min, max, or just.

<h3>upperCaseLetter()</h3>

Matches an upper case letter. This is the singular version of upperCaseLetters().

<h3>upperCaseLetters()</h3>

Matches an upper case letter. This method needs to be preceeded by min, max, or just.

<h3>dotAll()</h3>

Adds the dot all flag to the regular expression.

<h3>ignoreCase()</h3>

Adds the ignore case flag to the regular expression.

<h3>multiLine()</h3>

Adds the multiline flag to the regular expression.

<h3>global()</h3>

Adds the global flag to the regular expression.

<h3>sticky()</h3>

Adds the sticky flag to the regular expression.

<h3>unicode()</h3>

Adds the unicode flag to the regular expression.