﻿class RegExpBuilder {
  constructor() {
    this._flags = '';
    this._literal = '';

    this._clear();
  }

  _clear() {
    this._min = -1;
    this._max = -1;
    this._of = '';
    this._ofAny = false;
    this._groupName = '';
    this._from = '';
    this._notFrom = '';
    this._like = '';
    this._bypass = '';
    this._either = '';
    this._reluctant = false;
    this._captureName = '';
  }

  _flushState() {
    if (this._of || this._ofAny || this._groupName || this._from || this._notFrom || this._like || this._bypass) {
      const captureStart = this._getCapture();
      const captureEnd = captureStart ? ')' : '';
      const quantity = this._getQuantity();
      const character = this._getCharacter();
      const reluctant = this._reluctant ? '?' : '';
      this._literal += `${captureStart}${character}${quantity}${reluctant}${captureEnd}`;
      this._clear();
    }
  }

  _getCapture() {
    if (this._captureName) {
      return `(?<${this._captureName}>`;
    }
    return '';
  }

  _getQuantity() {
    if (this._min !== -1) {
      if (this._max !== -1) {
        if (this._min === 0 && this._max === 1) {
          return '?';
        }
        if (this._min === 1 && this._max === 1) {
          return '';
        }
        if (this._min === this._max) {
          return `{${this._min}}`;
        }
        return `{${this._min},${this._max}}`;
      }
      if (this._min === 0) {
        return '*';
      }
      if (this._min === 1) {
        return '+';
      }
      return `{${this._min},}`;
    }
    if (this._max === 1) {
      return '?';
    }
    return `{0,${this._max}}`;
  }

  _getCharacter() {
    if (this._of) {
      if (this._of.length > 1 && (this._min !== 1 || this._max !== 1)) {
        return `(${this._of})`;
      }
      return this._of;
    }
    if (this._ofAny) {
      return '.';
    }
    if (this._groupNumber > 0) {
      return `\\${this._groupNumber}`;
    }
    if (this._groupName) {
      return `\\k<${this._groupName}>`;
    }
    if (this._from) {
      return `[${this._from}]`;
    }
    if (this._notFrom) {
      return `[^${this._notFrom}]`;
    }
    if (this._like) {
      if (this._min === 1 && this._max === 1) {
        return this._like;
      }
      return `(${this._like})`;
    }
    if (this._bypass) {
      return this._bypass;
    }
  }

  getLiteral() {
    this._flushState();
    if (/^\([^?()][^()]+\)$/.test(this._literal)) {
      this._literal = this._literal.substring(1, this._literal.length - 1);
    }
    return this._literal;
  }
    
  getRegExp() {
    return new RegExp(this.getLiteral(), this._flags);
  }

  test(str) {
    return this.getRegExp().test(str);
  }

  exec(str) {
    return this.getRegExp().exec(str);
  }

  _addFlag(flag) {
    if (!this._flags.includes(flag)) {
      this._flags += flag;
    }
    return this;
  }

  dotAll() {
    return this._addFlag('s');
  }

  ignoreCase() {
    return this._addFlag('i');
  }
  
  multiLine() {
    return this._addFlag('m');
  }
  
  global() {
    return this._addFlag('g');
  }

  sticky() {
    return this._addFlag('y');
  }

  unicode() {
    return this._addFlag('u');
  }
  
  start() {
    this.just(1);
    this._bypass = '^';
    return this;
  }
    
  end() {
    this._flushState();
    this.just(1);
    this._bypass = '$';
    return this;
  }

  boundary() {
    this._flushState();
    this.just(1);
    this._bypass = '\\b';
    return this;
  }

  nonBoundary() {
    this._flushState();
    this.just(1);
    this._bypass = '\\B';
    return this;
  }
  
  either(pattern) {
    if (typeof pattern === 'string') {
      return this._eitherLike(new RegExpBuilder().just(1).of(pattern));
    }
    return this._eitherLike(pattern);
  }
  
  _eitherLike(builder) {
    this._flushState();
    this._either = builder.getLiteral();
    return this;
  }
    
  or(pattern) {
    if (typeof pattern === 'string') {
      return this._orLike(new RegExpBuilder().just(1).of(pattern));
    }
    return this._orLike(pattern);
  }
  
  _orLike(builder) {
    const either = this._either;
    const or = builder.getLiteral();
    if (!either) {
      this._literal = this._literal.substring(0, this._literal.length - 1);
      this._literal += `|${or})`;
    }
    else {
      this._literal += `(${either}|${or})`;
    }
    this._clear();
    return this;
  }
    
  just(count) {
    this._flushState();
    this._min = count;
    this._max = count;
    return this;
  }
    
  min(count) {
    this._flushState();
    this._min = count;
    return this;
  }
  
  max(count) {
    this._flushState();
    this._max = count;
    return this;
  }
  
  of(text) {
    this._of = this._sanitize(text);
    return this;
  }
    
  ofAny() {
    this._ofAny = true;
    return this;
  }
  
  ofGroup(label) {
    this._groupName = label;
    return this;
  }
  
  from(chars) {
    if (typeof chars !== 'string') {
      chars = chars.join('');
    }
    this._from = this._sanitize(chars);
    return this;
  }
    
  notFrom(chars) {
    if (typeof chars !== 'string') {
      chars = chars.join('');
    }
    this._notFrom = this._sanitize(chars);
    return this;
  }
  
  like(builder) {
    this._like = builder.getLiteral();
    return this;
  }
  
  reluctantly() {
    this._reluctant = true;
    return this;
  }
    
  ahead(pattern) {
    if (typeof pattern === 'string') {
      this._flushState();
      this._literal += `(?=${this._sanitize(pattern)})`;
      return this;  
    }
    return this._aheadLike(pattern);
  }

  _aheadLike(builder) {
    this._flushState();
    this._literal += `(?=${builder.getLiteral()})`;
    return this;
  }
  
  notAhead(pattern) {
    if (typeof pattern === 'string') {
      this._flushState();
      this._literal += `(?!${this._sanitize(pattern)})`;
      return this;  
    }
    return this._notAheadLike(pattern);
  }

  _notAheadLike(builder) {
    this._flushState();
    this._literal += `(?!${builder.getLiteral()})`;
    return this;
  }

  behind(pattern) {
    if (typeof pattern === 'string') {
      this._flushState();
      this._literal += `(?<=${this._sanitize(pattern)})`;
      return this;  
    }
    return this._aheadLike(pattern);
  }

  _behindLike(builder) {
    this._flushState();
    this._literal += `(?<${builder.getLiteral()})`;
    return this;
  }
  
  notBehind(pattern) {
    if (typeof pattern === 'string') {
      this._flushState();
      this._literal += `(?<!${this._sanitize(pattern)})`;
      return this;  
    }
    return this._notAheadLike(pattern);
  }

  _notBehindLike(builder) {
    this._flushState();
    this._literal += `(?<!${builder.getLiteral()})`;
    return this;
  }
  
  asGroup(name) {
    this._captureName = name;
    return this;
  }
    
  then(pattern) {
    if (typeof pattern === 'string') {
      return this.just(1).of(pattern);
    }
    return this.just(1).like(pattern);
  }
  
  find(pattern) {
    return this.then(pattern);
  }
  
  maybe(pattern) {
    if (typeof pattern === 'string') {
      return this.max(1).of(pattern);
    }
    return this.max(1).like(pattern);
  }

  withProp(prop) {
    this._bypass = `\\p{${prop}}`;
    return this;
  }

  withoutProp(prop) {
    this._bypass = `\\P{${prop}}`;
    return this;
  }
    
  any() {
    return this.just(1).ofAny();
  }
    
  lineBreak() {
    this._flushState();
    this.just(1);
    this._bypass = '\\r\\n|\\r|\\n';
    return this;
  }
  
  lineBreaks() {
    return this.like(new RegExpBuilder().lineBreak());
  }
  
  whiteSpace() {
    if (this._min === -1 && this._max === -1) {
      this._flushState();
      this.just(1);
      this._bypass = '\\s';
      return this;
    }
    this._bypass = '\\s';
    return this;
  }
    
  notWhiteSpace() {
    if (this._min === -1 && this._max === -1) {
      this._flushState();
      this.just(1);
      this._bypass = '\\S';
      return this;
    }
    this._bypass = '\\S';
    return this;
  }
  
  tab() {
    this._flushState();
    this.just(1);
    this._bypass = '\\t';
    return this;
  }
  
  tabs() {
    this._bypass = '\\t';
    return this;
  }
    
  digit() {
    this._flushState();
    this.just(1);
    this._bypass = '\\d';
    return this;
  }
  
  digits() {
    this._bypass = '\\d';
    return this;
  }
    
  letter() {
    this.just(1);
    this._from = 'A-Za-z';
    return this;
  }
  
  letters() {
    this._from = 'A-Za-z';
    return this;
  }
  
  lowerCaseLetter() {
    this.just(1);
    this._from = 'a-z';
    return this;
  }
    
  lowerCaseLetters() {
    this._from = 'a-z';
    return this;
  }
  
  upperCaseLetter() {
    this.just(1);
    this._from = 'A-Z';
    return this;
  }
  
  upperCaseLetters() {
    this._from = 'A-Z';
    return this;
  }
    
  _sanitize(text) {
    return text.replace(/([.*+?^=!:${}()|\[\]\/\\])/g, '\\$1');
  }
}

export default RegExpBuilder;
export const start = () => new RegExpBuilder().start();
export const end = () => new RegExpBuilder().end();
export const boundary = () => new RegExpBuilder().boundary();
export const nonBoundary = () => new RegExpBuilder().nonBoundary();
export const either = (pattern) => new RegExpBuilder().either(pattern);
export const just = (count) => new RegExpBuilder().just(count);
export const min = (count) => new RegExpBuilder().min(count);
export const max = (count) => new RegExpBuilder().max(count);
export const ahead = (pattern) => new RegExpBuilder().ahead(pattern);
export const notAhead = (pattern) => new RegExpBuilder().notAhead(pattern);
export const behind = (pattern) => new RegExpBuilder().behind(pattern);
export const notBehind = (pattern) => new RegExpBuilder().notBehind(pattern);
export const find = (pattern) => new RegExpBuilder().find(pattern);
export const maybe = (pattern) => new RegExpBuilder().maybe(pattern);
export const any = () => new RegExpBuilder().any();
export const lineBreak = () => new RegExpBuilder().lineBreak();
export const whiteSpace = () => new RegExpBuilder().whiteSpace();
export const notWhiteSpace = () => new RegExpBuilder().notWhiteSpace();
export const tab = () => new RegExpBuilder().tab();
export const digit = () => new RegExpBuilder().digit();
export const letter = () => new RegExpBuilder().letter();
export const lowerCaseLetter = () => new RegExpBuilder().lowerCaseLetter();
export const upperCaseLetter = () => new RegExpBuilder().upperCaseLetter();
