﻿import { find, start, just, either, min } from '../regexpbuilder.js';

class TestRunner {
  constructor(tests) {
    this._tests = tests;
    this._failed = 0;
  }

  run() {
    this._tests.forEach((test) => {
      test.run();
      this._printResult(test);
    });
    this._printSummary();
  }

  _printResult(test) {
    if (test.passed) {
      console.log(`PASS: ${test.name}`);
    }
    else {
      this._failed++;
      console.log(`FAIL: ${test.name}`);
    }
  }

  _printSummary() {
    console.log('');
    if (this._failed === 0) {
      console.log(`All ${this._tests.length} tests passed.`);
    }
    else {
      console.log(`${this._failed} of ${this._tests.length} tests failed.`);
    }
  }
}

class Test {
  constructor(name, test) {
    this.name = name;
    this.test = test;
    this.passed = true;
  }

  expect(result) {
    if (!result) {
      this.passed = false;
    }
  }

  run() {
    this.test(this);
  }
}

const tests = [];
tests.push(new Test('start', (test) => {
  const regex = start().just(1).of('p');
  
  test.expect(regex.test('p'));
  test.expect(!regex.test('qp'));
}));

tests.push(new Test('end', (test) => {
  const regex = just(1).of('p').end();
  
  test.expect(regex.test('p'));
  test.expect(!regex.test('pq'));
}));

tests.push(new Test('eitherLike orLike', (test) => {
  const p1 = just(1).of('p');
  const p2 = just(2).of('q');

  const regex = start()
    .either(p1)
    .or(p2)
    .end();
  
  test.expect(regex.test('p'));
  test.expect(regex.test('qq'));
  test.expect(!regex.test('pqq'));
  test.expect(!regex.test('qqp'));
}));

tests.push(new Test('orLike chain', (test) => {
  const p1 = just(1).of('p');
  const p2 = just(1).of('q');
  const p3 = just(1).of('r');

  const regex = either(p1).or(p2).or(p3);
  
  test.expect(regex.test('p'));
  test.expect(regex.test('q'));
  test.expect(regex.test('r'));
  test.expect(!regex.test('s'));
}));

tests.push(new Test('either or', (test) => {
  const regex = either('p').or('q');
  
  test.expect(regex.test('p'));
  test.expect(regex.test('q'));
  test.expect(!regex.test('r'));
}));

tests.push(new Test('just', (test) => {
  const regex = start()
    .just(3).of('p')
    .end();
  
  test.expect(regex.test('ppp'));
  test.expect(!regex.test('pp'));
  test.expect(!regex.test('pppp'));
}));

tests.push(new Test('min', (test) => {
  const regex = min(2).of('p');
  
  test.expect(regex.test('pp'));
  test.expect(regex.test('ppp'));
  test.expect(regex.test('ppppppp'));
  test.expect(!regex.test('p'));
}));

tests.push(new Test('max', (test) => {
  const regex = start()
    .max(3).of('p')
    .end();
  
  test.expect(regex.test('p'));
  test.expect(regex.test('pp'));
  test.expect(regex.test('ppp'));
  test.expect(!regex.test('pppp'));
  test.expect(!regex.test('pppppppp'));
}));

tests.push(new Test('min max', (test) => {
  const regex = start()
    .min(3).max(7).of('p')
    .end();
  
  test.expect(regex.test('ppp'));
  test.expect(regex.test('ppppp'));
  test.expect(regex.test('ppppppp'));
  test.expect(!regex.test('pp'));
  test.expect(!regex.test('p'));
  test.expect(!regex.test('pppppppp'));
  test.expect(!regex.test('pppppppppppp'));
}));

tests.push(new Test('of', (test) => {
  const regex = just(2).of('p p p ');
  
  test.expect(regex.test('p p p p p p '));
  test.expect(!regex.test('p p p p pp'));
}));

tests.push(new Test('ofAny', (test) => {
  const regex = just(3).ofAny();
  
  test.expect(regex.test('pqr'));
}));

tests.push(new Test('ofGroup', (test) => {
  const regex = just(3).of('p').asGroup('group')
    .just(1).of('q')
    .just(1).ofGroup('group');
  
  test.expect(regex.test('pppqppp'));
}));

tests.push(new Test('from', (test) => {
  const someLetters = ['p', 'q', 'r'];
  const regex = just(3).from(someLetters);
  
  test.expect(regex.test('ppp'));
  test.expect(regex.test('qqq'));
  test.expect(regex.test('ppq'));
  test.expect(regex.test('rqp'));
  test.expect(!regex.test('pyy'));
}));

tests.push(new Test('from range', (test) => {
  const someLetters = ['p-r', 'v'];
  const regex = just(3).from(someLetters);
  
  test.expect(regex.test('ppp'));
  test.expect(regex.test('qqq'));
  test.expect(regex.test('pqv'));
  test.expect(regex.test('prr'));
  test.expect(!regex.test('pr-'));
}));

tests.push(new Test('notFrom', (test) => {
  const someLetters = ['p', 'q', 'r'];
  const regex = just(3).notFrom(someLetters);
  
  test.expect(regex.test('lmn'));
  test.expect(!regex.test('mnq'));
}));

tests.push(new Test('like', (test) => {
  const pattern = min(1).of('p')
    .min(2).of('q');
  
  const regex = just(2).like(pattern);
  
  test.expect(regex.test('pqqpqq'));
  test.expect(!regex.test('qppqpp'));
}));

tests.push(new Test('reluctantly', (test) => {
  const regex = just(2).of('p')
    .min(2).ofAny().reluctantly()
    .just(2).of('p');
  
  test.expect(regex.exec('pprrrrpprrpp')[0] === 'pprrrrpp');
}));

tests.push(new Test('ahead', (test) => {
  const regex = just(1).of('dart')
    .ahead(just(1).of('lang'));
  
  test.expect(regex.exec('dartlang')[0] === 'dart');
  test.expect(!regex.test('dartpqr'));
}));

tests.push(new Test('notAhead', (test) => {
  const regex = just(1).of('dart')
    .notAhead(just(1).of('pqr'));
  
  test.expect(regex.test('dartlang'));
  test.expect(!regex.test('dartpqr'));
}));

tests.push(new Test('asGroup', (test) => {
  const regex = min(1).max(3).of('p')
    .just(1).of('dart').asGroup('group')
    .just(1).from(['p', 'q', 'r']);
  
  test.expect(regex.exec('pdartq').groups.group === 'dart');
}));

tests.push(new Test('phone number', (test) => {
  const regex = start()
    .find('(')
    .min(1).digits()
    .then(')')
    .then(' ')
    .just(4).digits()
    .then(' ')
    .just(4).digits()
    .end();
  
  test.expect(regex.test('(07) 3782 9481'));
  test.expect(!regex.test('(07 3782 9481'));
  test.expect(!regex.test('(07) 37829481'));
  test.expect(!regex.test('(07) 37824 9481'));
}));

tests.push(new Test('complex', (test) => {
  const p1 = min(3).ofAny();
  const p2 = just(2).from(['e', 'h', 'l'])
    .either('house')
    .or('home')
    .or('household');

  const regex = start()
    .min(2).like(p1).reluctantly()
    .then('the cure')
    .either(p1)
    .or(p2)
    .just(3).like(p2);
  
  test.expect(regex.test('fhjehfthe cureeehomeeehouseholdllhousellhome'));
}));

tests.push(new Test('boundary', (test) => {
  const regex = find('ure').boundary();
  
  test.expect(!regex.test('Thecureisaband'));
  test.expect(regex.test('The cure is a band.'));
}));

tests.push(new Test('nonBoundary', (test) => {
  const regex = find('ure').nonBoundary();
  
  test.expect(regex.test('Thecureisaband'));
  test.expect(!regex.test('The cure is a band.'));
}));

tests.push(new Test('withProp', (test) => {
  const regex = min(2).withProp('Emoji').unicode();
  
  test.expect(!regex.test('There are no emojis here.'));
  test.expect(regex.test('This has emojis 😂😍'));
}));

const testRunner = new TestRunner(tests);
testRunner.run();
